/*
 * LyMapBeahvior.java
 * Copyright (c) 2008, Lyria.
 * All rights reserved.
 */

package leon.app.behavior;

import leon.app.*;
import leon.app.behaviorinterface.*;
import leon.control.*;
import leon.data.*;
import leon.info.*;
import leon.misc.*;
import leon.view.*;


/**
 * Default application behavior for map controller interface.
 *
 * @author  Lyria S.A.
 * @version $Id: LyMapBehavior.java,v 6.9 2007/12/05 09:01:02 jls Exp $
 * @see     leon.control.LyMapController
 * @see     leon.app.behaviorinterface.LyMapBehaviorInterface
 */
public class LyMapBehavior extends LyPanelBehavior
	implements LyMapBehaviorInterface
{
	public static final String __VERSION =
		"$Id: LyMapBehavior.java,v 6.9 2007/12/05 09:01:02 jls Exp $";

    public static int       IMAGE_HEIGHT = 100;
    
	/**
	 * Constructor.
	 */
	public LyMapBehavior()
	{
	}

	/**
	 * Gets the application objects list (which are {@link leon.data.LyObject LyObject} instances),
	 * that must be represented as <u>node</u> objects.<br>
	 * <b>IMPORTANT:</b> if the returned list is a dynamic workspace, the mapController will notify
	 * for node updates in {@link #updateNode(LyObject, LyMapNode, LyMapController)}. Association
	 * between application object and graphical object is made in the {@link #buildNode(LyObject,
	 * LyMapController)} method.<br />
	 * <i>The default implementation</i> returns all objects of the controller classInfo depending
	 * on the current context of the controller.
	 *
	 * @param  controller current LyMapController instance
	 * @return the list of node objects, or null if the map does not have nodes
	 * @see    #buildNode(LyObject, LyMapController)
	 * @see    #updateNode(LyObject, LyMapNode, LyMapController)
	 */
	public LyObjectList getNodeObjects(LyMapController controller)
	{
		LySession   session = controller.getSession();
		LyWorkSpace ws = null;
		LyWorkSpace context = controller.getContext();
		LyClassInfo classInfo = controller.getClassInfo();

		if (context != null)
		{
			// if classInfo is not null, building a mixed workspace
			if (classInfo != null)
				// Return a new dynamic workspace based on context
				ws = (LyWorkSpace)context.getList(session, classInfo, true);
		}
		else
		{
			// Return all the objects of this class info
			if (classInfo != null)
				// TODO : optimize the loaded fieldinfos.
				ws = new LyClassWorkSpace(session, classInfo, true);
		}

		if (ws != null)
			ws.setSortByName(true);
		return ws;
	}

	/**
	 * Gets the list of objects of the application (which are {@link leon.data.LyObject LyObject}
	 * instances), that must be represented as <u>link</u> objects.<br />
	 * <b>IMPORTANT:</b> if the returned list is a dynamic workspace, the mapController will notify
	 * for link updates in {@link #updateLink(LyObject, LyMapLink, LyMapController)}. Association
	 * between application object and graphical object is made in the {@link #buildLink(LyObject,
	 * LyMapController)} method.<br />
	 * <i>The default implementation</i> returns null. By default, no link object is created.
	 *
	 * @param  controller current LyMapController instance
	 * @return the list of links objects, or null if the map does not have links
	 * @see    #buildLink(LyObject, LyMapController)
	 * @see    #updateLink(LyObject, LyMapLink, LyMapController)
	 */
	public LyObjectList getLinkObjects(LyMapController controller)
	{
		return null;
	}

	/**
	 * Gets the list of objects of the application (which are {@link leon.data.LyObject LyObject}
	 * instances), that must be represented as <u>region</u> objects.<br />
	 * <b>IMPORTANT:</b> if the returned list is a dynamic workspace the mapController will notify
	 * for region updates in {@link #updateRegion(LyObject, LyMapRegion, LyMapController)}.
	 * Association between application object and graphical object is made in {@link
	 * #buildRegion(LyObject, LyMapController)} method.<br />
	 * <i>The default implementation</i> returns null. By Default, no region objects is created.
	 *
	 * @param  controller current LyMapController instance
	 * @return the list of region objects, or null if the map does not have regions
	 * @see    #buildRegion(LyObject, LyMapController)
	 * @see    #updateRegion(LyObject, LyMapRegion, LyMapController)
	 */
	public LyObjectList getRegionObjects(LyMapController controller)
	{
		return null;
	}

	/**
	 * Builds a node object instance from the given object.<br />
	 * <u>Note:</u> Node status (colors, borders, cross...) should be set in {@link
	 * #updateNode(LyObject, LyMapNode, LyMapController)}.<br />
	 * <i>The default implementation</i> creates a {@link LyMapNode map node} with the given node id
	 * and sets its style as an {@link LyMapNode#STYLE_IMAGE image}.
	 *
	 * @param  nodeObject application object, usually a LyObject instance but not mandatory
	 * @param  controller current LyMapController instance
	 * @return the built node view object.
	 */
	public LyMapNode buildNode(LyObject nodeObject, LyMapController controller)
	{
		if (nodeObject == null)
			return null;

		LyMapNode node = new LyMapNode();

		node.setId(nodeObject.getId());
		node.setStyle(LyMapNode.Style.IMAGE);

		return (node);
	}

	/**
	 * Builds a link object instance from the application object.<br />
	 * <u>Note:</u> Link status (colors, icons, ...) should be set in {@link #updateLink(LyObject,
	 * LyMapLink, LyMapController)} .<br />
	 * <i>The default implementation</i> does nothing and returns null.
	 *
	 * @param  linkObject application object, usually LyObject instance but not mandatory
	 * @param  controller current LyMapController instance
	 * @return the built link view object
	 */
	public LyMapLink buildLink(LyObject linkObject, LyMapController controller)
	{
		return null;
	}

	/**
	 * Builds a region object instance from the application object.<br />
	 * <u>Note:</u> Region status (colors, icons, ...) should be set in {@link
	 * #updateRegion(LyObject, LyMapRegion, LyMapController)} .<br />
	 * <i>The default implementation</i> does nothing and returns null.
	 *
	 * @param  regionObject application object, usually LyObject instance but not mandatory
	 * @param  controller   current LyMapController instance
	 * @return the built region view object
	 */
	public LyMapRegion buildRegion(LyObject regionObject, LyMapController controller)
	{
		return null;
	}

	/**
	 * Updates a the given mapNode object status (colors, label, border, style, ...) with the given
	 * nodeObject properties. The controller invokes this method when the given nodeObject is
	 * modified.<br />
	 * <i>The default implementation</i> updates the label, the image associated to the node and the
	 * hidden property.
	 *
	 * @param nodeObject application object (reference)
	 * @param mapNode    view objects (need update)
	 * @param controller current LyMapController instance
	 */
	public void updateNode(LyObject nodeObject, LyMapNode mapNode, LyMapController controller)
	{
		if ((nodeObject == null) || (mapNode == null))
			return;

		// Set the label with the object name and the image with
		// the class icon
		mapNode.setLabel(controller.getName(nodeObject));
		mapNode.setImage(getImage(controller, nodeObject));
		mapNode.setHidden(nodeObject.isHidden());
	}

    /**
     * Gets image corresponding to an object.
     * @return Image that represents the given object.
     */
    public String getImage(LyMapController map, LyObject object)
    {
        // Try to find a file fieldInfo with image type
        LyClassInfo     classInfo = object.getClassInfo();
        LyFieldInfo     fieldInfo;
        int             i, n = classInfo.getFieldInfoCount();
        LyFileValue     value;
        String          image = null;
        LyEnvironment   env = map.getEnvironment();
        
        for (i = 0; i < n; i++)
        {
            fieldInfo = classInfo.getFieldInfo(i);
            if ((fieldInfo instanceof LyFileFieldInfo)
                && (((LyFileFieldInfo)fieldInfo).getType() == LyFileFieldInfo.Type.IMAGE))
            {
                value = object.getFileValue((LyFileFieldInfo)fieldInfo);
                if (!value.isNull())
                    image = ((LyFileFieldInfo)fieldInfo).getUrl(env, value.getString());
                break;
            }
        }
        
        if (image != null)
        {
            if (image.startsWith("file://"))
                image = image.substring(7);
            
            // adding the size information
            if (IMAGE_HEIGHT > 0)
            {
                LyString string = env.getString();

                image += string.VISIBLE_SEP3+ IMAGE_HEIGHT + string.VISIBLE_SEP3_BIS;
            }
        }
        else
        {
            // Retrieve the image corresponding to the object classInfo  
           image = map.getImageFromPrefix(classInfo.getFilePrefix(),
                    LyController.DEFAULT_IMAGE_FORMAT, 0);
        }
        
        return image;
    }
    
	/**
	 * Updates a the given mapLink object status (colors, label, border, style, ...) with the given
	 * linkObject properties. The controller invokes this method when the given linkObject is
	 * modified.<br />
	 * <i>The default implementation</i> does nothing.
	 *
	 * @param linkObject application object (reference)
	 * @param mapLink    view objects (need update)
	 * @param controller current LyMapController instance
	 */
	public void updateLink(LyObject linkObject, LyMapLink mapLink, LyMapController controller)
	{
	}

	/**
	 * Updates a the given mapRegion object status (colors, label, border, style, ...) with the
	 * given regionObject properties. The controller invokes this method when the given regionObject
	 * is modified.<br />
	 * <i>The default implementation</i> does nothing.
	 *
	 * @param regionObject application object (reference)
	 * @param mapRegion    view objects (need update)
	 * @param controller   current LyMapController instance
	 */
	public void updateRegion(LyObject regionObject, LyMapRegion mapRegion,
		LyMapController controller)
	{
	}

	/**
	 * This method is invoked by the controller when the given mapNode is selected on the map.<br />
	 * <i>The default implementation</i> does nothing.
	 *
	 * @param nodeObject application object (reference)
	 * @param mapNode    view objects
	 * @param controller current LyMapController instance
	 */
	public void selectNode(LyObject nodeObject, LyMapNode mapNode, LyMapController controller)
	{
	}

	/**
	 * This method is invoked by the controller when the given mapNode is unselected on the map.
	 * <br />
	 * <i>The default implementation</i> does nothing.
	 *
	 * @param nodeObject application object (reference)
	 * @param mapNode    view objects
	 * @param controller current LyMapController instance
	 */
	public void unselectNode(LyObject nodeObject, LyMapNode mapNode, LyMapController controller)
	{
	}

	/**
	 * This method is invoked by the controller when all nodes are unselected on the map.<br />
	 * <i>The default implementation</i> does nothing.
	 *
	 * @param controller current LyMapController instance
	 */
	public void unselectNodes(LyMapController controller)
	{
	}

	/**
	 * This method is invoked by the controller when selected nodes on the map have been moved.
	 * <br />
	 * <i>The default implementation</i> does nothing, but children classes may set coordinates in
	 * the appObject for instance.
	 *
	 * @param controller current mapController instance
	 */
	public void selectionMoved(LyMapController controller)
	{
	}

	/**
	 * This method is invoked by the controller to invoke the default action when an object is
	 * activated on the map (e.g. by double click with graphical modality).<br />
	 * <i>The default implementation</i> does nothing and return false.
	 *
	 * @param      nodeObject application object (may be null)
	 * @param      mapObject  view objects (selected for action)
	 * @param      controller current LyMapController instance
	 * @return     true if the action was handled, false otherwise
	 * @deprecated replaced by {@link #invokeDefaultAction(LyObject, LyComponent, int, int,
	 *             LyMapController)}
	 */
	public boolean invokeDefaultAction(LyObject nodeObject, LyComponent mapObject,
		LyMapController controller)
	{
		return false;
	}

	/**
	 * This method is invoked by the controller to invoke the default action when an object is
	 * activated on the map (e.g. by double click with graphical modality).<br />
	 * <i>The default implementation</i> does nothing and return false.
	 *
	 * @param  nodeObject application object (may be null)
	 * @param  mapObject  view objects (selected for action)
	 * @param  x          the x position of the event on the map
	 * @param  y          the y position of the event on the map
	 * @param  controller current LyMapController instance
	 * @return true if the action was handled, false otherwise
	 */
	public boolean invokeDefaultAction(LyObject nodeObject, LyComponent mapObject, int x, int y,
		LyMapController controller)
	{
		return false;
	}

	/**
	 * This method is invoked when the view is ready to be shown. May be overriden to add extra
	 * components.<br />
	 * <i>The default implementation</i> sets the map layout to a matrix view and disables the
	 * viewer.
	 *
	 * @param controller current mapController instance
	 */
	public void completeMap(LyMapController controller)
	{
		LyMap map = controller.getMapComponent();

		// Set the layout
		if ((map != null) && (map.getLayout() == null))
		{
			// Create a default layout.
			LyLayout layout = new LyLayout();

			layout.setStyle(LyLayout.Style.MATRIX);
			layout.setDeltaNode(20);
			layout.setDeltaBorder(10);
			layout.setHAlign(LyLayout.HAlign.CENTER);
			layout.setVAlign(LyLayout.VAlign.TOP);
			layout.setFill(LyLayout.Fill.HORIZONTAL);

			map.setLayout(layout);
			map.setViewer(false);
		}
	}

	/**
	 * This method is invoked when the content of the map has been modified.<br />
	 * <i>The default implementation</i> does nothing.
	 *
	 * @param controller current mapController instance
	 * @param map        current map instance
	 * @param content    new map content instance
	 */
	public void setMapContent(LyMapController controller, LyMap map, LyMapContent content)
	{
	}

	/**
	 * Frees all resources.<br />
	 * <i>The default implementation</i> does nothing.
	 *
	 * @param controller current mapController instance
	 */
	public void free(LyMapController controller)
	{
	}

	/**
	 * Method invoked at the beginning of the given object drag.<br />
	 * <i>The default implementation</i> keep in the controller session memory that the object is
	 * dragged and log a trace.
	 *
	 * @param controller    current LyMapController instance
	 * @param draggedObject dragged object
	 */
	public void dragStart(LyMapController controller, LyObject draggedObject)
	{
		LyEnvironment environment = controller.getEnvironment();
		LySession     session = controller.getSession();

		session.addValue(LySession.DRAGGED_OBJECT, draggedObject);
		environment.logTrace("Drag is starting with object: " + draggedObject);
	}

	/**
	 * Method called during a drag-and-drop, when the given object is dragged over the map
	 * component.<br />
	 * <i>The default implementation</i> only logs a trace.
	 *
	 * @param controller    current LyMapController instance
	 * @param draggedObject the dragged object
	 * @param x             The x position of the drag event
	 * @param y             The y position of the drag event
	 */
	public void dragOver(LyMapController controller, LyObject draggedObject, int x, int y)
	{
		LyEnvironment environment = controller.getEnvironment();

		environment.logTrace("Drag is over the map with object: " + draggedObject + " at position "
			+ x + ", " + y);
	}

	/**
	 * Method called when the given dragged object is dropped on the given dropped object.<br />
	 * <i>The default implementation</i> only logs a trace.
	 *
	 * @param controller    current LyMapController instance
	 * @param droppedObject object on where the dragged object is dropped
	 * @param draggedObject the dragged object
	 * @param x             The x position of drop event
	 * @param y             The y position of drop event
	 */
	public void drop(LyMapController controller, LyObject droppedObject, LyObject draggedObject,
		int x, int y)
	{
		LyEnvironment environment = controller.getEnvironment();

		environment.logTrace("Object: " + draggedObject + " is dropped on object " + droppedObject
			+ " at position " + x + ", " + y);
	}

	/**
	 * Method called at the end of a drag-and-drop.<br />
	 * <i>The default implementation</i> only logs a trace about the given drag-and-drop type.
	 *
	 * @param controller current LyMapController instance
	 * @param type       drag-and-drop type ({@link org.eclipse.swt.dnd.DND#DROP_COPY} or {@link
	 *                   org.eclipse.swt.dnd.DND#DROP_MOVE})
	 */
	public void dragFinished(LyMapController controller, int type)
	{
		// String dropType;
		// switch (type)
		// {
		// case DND.DROP_COPY:
		// dropType = "Copy (Ctrl + Click)";
		// break;
		// case DND.DROP_MOVE:
		// dropType = "Move (Shift + Click) or default";
		// break;
		// case DND.DROP_LINK:
		// dropType = "Link (Ctrl + Shift + Click)";
		// break;
		// default:
		// dropType = "None";
		// break;
		// }

		LyEnvironment environment = controller.getEnvironment();

		environment.logTrace("The drag-and-drop type is: " + type);
	}
} // end class LyMapBehavior
